import { takeLeading, call, put, select } from "redux-saga/effects";
import { ACTIONS } from "../actions";
import AuthAPI from "../apis/auth";
import {
  APP_STATE,
  ControlsType,
  KeystrokeResultType,
  Reducers,
  UpdatePasswordPayload,
} from "../types";

function* updateKeystrokeSettings({
  payload,
}: {
  type: typeof ACTIONS.UPDATE_KEYSTROKE_SETTINGS;
  payload: ControlsType;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    yield call(AuthAPI.updateControls, payload);
    const keystrokeResult: KeystrokeResultType = yield select(
      (state: Reducers) => state.auth.keystrokeResult
    );
    yield put({
      type: ACTIONS.SET_AUTH,
      payload: { keystrokeResult: { ...keystrokeResult, controls: payload } },
    });
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
  } catch (error) {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED, msg: error },
    });
  }
}

function* changePassword({
  payload,
}: {
  type: typeof ACTIONS.CHANGE_PASSWORD;
  payload: UpdatePasswordPayload;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    yield call(AuthAPI.updatePassword, payload);
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
  } catch (error) {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED, msg: error },
    });
  }
}

export default function* authSaga() {
  yield takeLeading(ACTIONS.UPDATE_KEYSTROKE_SETTINGS, updateKeystrokeSettings);
  yield takeLeading(ACTIONS.CHANGE_PASSWORD, changePassword);
}
